/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.net.protocol.classpath;

import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.util.Hashtable;

public class Handler extends URLStreamHandler {

  private final Class<?> _class;

  public Handler() {
    this._class = getClass();
  }

  public Handler(Class<?> _class) {
    this._class = _class;
  }

  @Override
  protected URLConnection openConnection(URL url) throws IOException {
    return new ClassPathURLConnection(url, _class);
  }

  public static void register() throws Exception {
    final Field handlersField = URL.class.getDeclaredField("handlers");
    handlersField.setAccessible(true);
    @SuppressWarnings("unchecked")
    final Hashtable<String, URLStreamHandler> handlers = (Hashtable<String, URLStreamHandler>) handlersField
        .get(null);
    handlers.put("classpath", new Handler());

  }
}
